/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 *    Copyright (c) [2017] Payara Foundation and/or its affiliates. All rights reserved.
 * 
 *     The contents of this file are subject to the terms of either the GNU
 *     General Public License Version 2 only ("GPL") or the Common Development
 *     and Distribution License("CDDL") (collectively, the "License").  You
 *     may not use this file except in compliance with the License.  You can
 *     obtain a copy of the License at
 *     https://github.com/payara/Payara/blob/master/LICENSE.txt
 *     See the License for the specific
 *     language governing permissions and limitations under the License.
 * 
 *     When distributing the software, include this License Header Notice in each
 *     file and include the License file at glassfish/legal/LICENSE.txt.
 * 
 *     GPL Classpath Exception:
 *     The Payara Foundation designates this particular file as subject to the "Classpath"
 *     exception as provided by the Payara Foundation in the GPL Version 2 section of the License
 *     file that accompanied this code.
 * 
 *     Modifications:
 *     If applicable, add the following below the License Header, with the fields
 *     enclosed by brackets [] replaced by your own identifying information:
 *     "Portions Copyright [year] [name of copyright owner]"
 * 
 *     Contributor(s):
 *     If you wish your version of this file to be governed by only the CDDL or
 *     only the GPL Version 2, indicate your decision by adding "[Contributor]
 *     elects to include this software in this distribution under the [CDDL or GPL
 *     Version 2] license."  If you don't indicate a single choice of license, a
 *     recipient has the option to distribute your version of this file under
 *     either the CDDL, the GPL Version 2 or to extend the choice of license to
 *     its licensees as provided above.  However, if you add GPL Version 2 code
 *     and therefore, elected the GPL Version 2 license, then the option applies
 *     only if the new code is made subject to such option by the copyright
 *     holder.
 */
package fish.payara.nucleus.executorservice.admin;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.util.ColumnFormatter;
import fish.payara.nucleus.executorservice.PayaraExecutorServiceConfiguration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import javax.inject.Inject;
import org.glassfish.api.Param;
import org.glassfish.api.admin.AdminCommand;
import org.glassfish.api.admin.AdminCommandContext;
import org.glassfish.api.admin.ExecuteOn;
import org.glassfish.api.admin.RestEndpoint;
import org.glassfish.api.admin.RestEndpoints;
import org.glassfish.api.admin.RuntimeType;
import org.glassfish.hk2.api.PerLookup;
import org.glassfish.internal.api.Target;
import org.jvnet.hk2.annotations.Service;

/**
 *
 * @author Andrew Pielage
 */
@Service(name = "get-payara-executor-service-configuration")
@PerLookup
@ExecuteOn(RuntimeType.DAS)
@RestEndpoints({
    @RestEndpoint(configBean = PayaraExecutorServiceConfiguration.class,
            opType = RestEndpoint.OpType.GET,
            path = "get-payara-executor-service-configuration",
            description = "Gets the Payara Executor Service Configuration")
})
public class GetPayaraExecutorServiceConfigurationCommand implements AdminCommand {

    @Inject
    private Target targetUtil;
    
    @Param(name = "target", optional = true, defaultValue = "server-config")
    String target;
    
    private static final String[] HEADERS = {"Thread Pool Executor Core Pool Size", 
            "Thread Pool Executor Max Pool Size", "Thread Pool Executor Keep Alive Time", 
            "Thread Pool Executor Keep Alive Time Unit", "Thread Pool Executor Queue Size", 
            "Scheduled Thread Pool Executor Core Pool Size"};
    
    @Override
    public void execute(AdminCommandContext acc) {
        Config configNode = targetUtil.getConfig(target);
        PayaraExecutorServiceConfiguration payaraExecutorServiceConfiguration = configNode.getExtensionByType(
                PayaraExecutorServiceConfiguration.class);

        ColumnFormatter columnFormatter = new ColumnFormatter(HEADERS);
        Object values[] = {payaraExecutorServiceConfiguration.getThreadPoolExecutorCorePoolSize(),
                payaraExecutorServiceConfiguration.getThreadPoolExecutorMaxPoolSize(),
                payaraExecutorServiceConfiguration.getThreadPoolExecutorKeepAliveTime(),
                payaraExecutorServiceConfiguration.getThreadPoolExecutorKeepAliveTimeUnit(),
                payaraExecutorServiceConfiguration.getThreadPoolExecutorQueueSize(),
                payaraExecutorServiceConfiguration.getScheduledThreadPoolExecutorCorePoolSize()
        };

        columnFormatter.addRow(values);

        Map<String, Object> map = new HashMap<>();
        Properties extraProps = new Properties();
        
        map.put("threadPoolExecutorCorePoolSize", values[0]);
        map.put("threadPoolExecutorMaxPoolSize", values[1]);
        map.put("threadPoolExecutorKeepAliveTime", values[2]);
        map.put("threadPoolExecutorKeepAliveTimeUnit", values[3]);
        map.put("threadPoolExecutorQueueSize", values[4]);
        map.put("scheduledThreadPoolExecutorCorePoolSize", values[5]);
        extraProps.put("payaraExecutorServiceConfiguration", map);

        acc.getActionReport().setExtraProperties(extraProps);
        acc.getActionReport().setMessage(columnFormatter.toString());
    } 
}
